import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:og1_dashboard/src/common/enums/medium.dart';
import 'package:og1_dashboard/src/common/models/app_icon.dart';
import 'package:og1_dashboard/src/services/medium_service.dart';
import 'package:og1_dashboard/src/services/page_service.dart';
import 'package:og1_dashboard/src/widgets/page_grid.dart';
import 'package:og1_dashboard/src/widgets/search_widget.dart';
import 'package:og1_system/og1_system.dart';
import 'package:smooth_page_indicator/smooth_page_indicator.dart';

class Og1Dashboard extends StatefulWidget {
  final List<AppIcon> appIcons;
  final bool displayNavArrows;
  final String? hintSearch;

  const Og1Dashboard({
    Key? key,
    required this.appIcons,
    this.displayNavArrows = true,
    this.hintSearch,
  }) : super(key: key);

  @override
  _Og1DashboardState createState() => _Og1DashboardState();
}

class _Og1DashboardState extends State<Og1Dashboard> {
  final PageController _pageController = PageController(initialPage: 0);
  final TextEditingController _searchController = TextEditingController();
  List<PageGrid> gridViews = [];
  final String emptyDashboardText = 'No Apps available!';
  late List<AppIcon> filteredAppIcons;
  String searchText = '';
  Size size = Size(window.physicalSize.width / window.devicePixelRatio,
      window.physicalSize.height / window.devicePixelRatio);

  @override
  void initState() {
    filteredAppIcons = widget.appIcons;
    gridViews = PageService().generateGridViews(filteredAppIcons, size);
    _searchController.addListener(() {
      if (searchText != _searchController.text) {
        searchText = _searchController.text;
        filteredAppIcons = _searchApps(_searchController.text, widget.appIcons);
        gridViews = PageService().generateGridViews(filteredAppIcons, size);
        if (gridViews.isNotEmpty && _pageController.hasClients) {
          _pageController.jumpToPage(0);
        }
        setState(() {
          gridViews;
          searchText;
          filteredAppIcons;
        });
      }
    });
    super.initState();
  }

  @override
  void didUpdateWidget(covariant Og1Dashboard oldWidget) {
    filteredAppIcons = _searchApps(_searchController.text, widget.appIcons);
    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    return ResponsiveBuilder(builder: (context, sizingInformation) {
      size = sizingInformation.localWidgetSize;
      gridViews = PageService().generateGridViews(filteredAppIcons, size);

      return _buildOg1Dashboard();
    });
  }

  Column _buildOg1Dashboard() {
    return Column(mainAxisAlignment: MainAxisAlignment.center, children: [
      SizedBox(
          width: size.width < 600 ? size.width : 300,
          child: SearchWidget(
            searchController: _searchController,
            hintSearch: widget.hintSearch,
          )),
      Expanded(
          child: ScrollConfiguration(
              behavior: ScrollConfiguration.of(context).copyWith(dragDevices: {
                PointerDeviceKind.touch,
                PointerDeviceKind.mouse
              }),
              child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceAround,
                  children: _buildMediumDashboard()))),
      SmoothPageIndicator(
        controller: _pageController,
        count: gridViews.isNotEmpty ? gridViews.length : 1,
        effect: const JumpingDotEffect(),
        onDotClicked: (int pageIndex) =>
            MediumService().getMedium(size) == Medium.mobile ||
                    MediumService().getMedium(size) == Medium.tablet
                ? true
                : _animateToPage(pageIndex),
      ),
    ]);
  }

  List<Widget> _buildMediumDashboard() {
    if (gridViews.isEmpty) {
      return [Text(emptyDashboardText)];
    } else if (!widget.displayNavArrows ||
        MediumService().getMedium(size) == Medium.mobile ||
        MediumService().getMedium(size) == Medium.tablet) {
      return [_buildDashboard()];
    }
    return [
      _buildArrowButton(Icons.arrow_left_rounded, () => _previousPage()),
      _buildDashboard(),
      _buildArrowButton(Icons.arrow_right_rounded, () => _nextPage()),
    ];
  }

  Widget _buildDashboard() {
    return Expanded(
        flex: 18,
        child: PageView(
          controller: _pageController,
          children: gridViews,
        ));
  }

  Widget _buildArrowButton(IconData iconData, VoidCallback? onPressed) =>
      Expanded(
          flex: 2,
          child: IconButton(
            padding: const EdgeInsets.all(0),
            iconSize: 50,
            splashRadius: 20,
            icon: Icon(iconData),
            color: Colors.black,
            onPressed: onPressed,
          ));

  void _nextPage() {
    int? currentPage = _pageController.page?.toInt();
    if (currentPage != null) {
      _animateToPage(currentPage + 1);
    }
  }

  void _previousPage() {
    int? currentPage = _pageController.page?.toInt();
    if (currentPage != null) {
      _animateToPage(currentPage - 1);
    }
  }

  void _animateToPage(int pageIndex) => _pageController.animateToPage(pageIndex,
      duration: const Duration(milliseconds: 400), curve: Curves.easeIn);

  List<AppIcon> _searchApps(
      String searchControllerText, List<AppIcon> appIcons) {
    return appIcons
        .where((appIcon) => appIcon.title
            .toLowerCase()
            .contains(searchControllerText.toLowerCase()))
        .toList();
  }

  @override
  void dispose() {
    _pageController.dispose();
    _searchController.dispose();
    super.dispose();
  }
}
